@@ -1,13 +1,12 @@
 #!/usr/bin/perl
 use strict;
 
-#VERSION,2.1.4
-# $Id: nikto.pl 632 2011-02-19 02:49:31Z sullo $
+#VERSION,2.1.5
+# $Id$
 use Getopt::Long;
+use Time::Local;
 Getopt::Long::Configure('no_ignore_case');
 
-# use LW2;                   ### Change this line to use a different installed version
-
 ###############################################################################
 #                               Nikto                                         #
 ###############################################################################
@@ -24,8 +23,8 @@ Getopt::Long::Configure('no_ignore_case');
 #  GNU General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
-#  along with this program; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+#  along with this program; if not, write to
+#  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 # Contact Information:
 #     Sullo (sullo@cirt.net)
@@ -40,52 +39,31 @@ Getopt::Long::Configure('no_ignore_case');
 
 # global var/definitions
 use vars qw/$TEMPLATES %CLI %VARIABLES %TESTS/;
-use vars qw/%NIKTO %CONFIGFILE %request %result %COUNTERS %db_extensions/;
-use vars qw/@RESULTS @PLUGINS @DBFILE @REPORTS %CACHE %CONTENTSEARCH/;
+use vars qw/%NIKTO %CONFIGFILE %COUNTERS %db_extensions/;
+use vars qw/@RESULTS @PLUGINS @DBFILE @REPORTS %CONTENTSEARCH/;
 
 # setup
 $COUNTERS{'scan_start'}  = time();
 $VARIABLES{'DIV'}        = "-" x 75;
 $VARIABLES{'name'}       = "Nikto";
-$VARIABLES{'version'}    = "2.1.4";
+$VARIABLES{'version'}    = "2.1.5";
 $VARIABLES{'configfile'} = "/etc/nikto.conf";    ### Change if it's having trouble finding it
 
 # signal trap so we can close down reports properly
 $SIG{'INT'} = \&safe_quit;
 
-# read just the --config option
-{
-    my %optcfg;
-    Getopt::Long::Configure('pass_through', 'noauto_abbrev');
-    GetOptions(\%optcfg, "config=s");
-    Getopt::Long::Configure('nopass_through', 'auto_abbrev');
-    if (defined $optcfg{'config'}) { $VARIABLES{'configfile'} = $optcfg{'config'}; }
-}
-
-# Read the config files in order
-my ($error, $home);
-my $config_exists = 0;
-$error = load_config("$VARIABLES{'configfile'}");
-$config_exists = 1 if ($error eq "");
-
-# Guess home directory -- to support Windows
-foreach my $var (split(/ /, "HOME USERPROFILE")) {
-    $home = $ENV{$var} if ($ENV{$var});
-}
-$error         = load_config("$home/nikto.conf");
-$config_exists = 1 if ($error eq "");
-$error         = load_config("nikto.conf");
-$config_exists = 1 if ($error eq "");
-
-if ($config_exists == 0) {
-    die "- Could not find a valid nikto config file\n";
-}
-
+config_init();
 setup_dirs();
 require "$CONFIGFILE{'PLUGINDIR'}/nikto_core.plugin";
 nprint("T:" . localtime($COUNTERS{'scan_start'}) . ": Starting", "d");
-require "$CONFIGFILE{'PLUGINDIR'}/nikto_single.plugin";
 require "$CONFIGFILE{'PLUGINDIR'}/LW2.pm";
+require "$CONFIGFILE{'PLUGINDIR'}/JSON-PP.pm";
+$VARIABLES{'GMTOFFSET'} = gmt_offset();
+
+# use LW2;                   ### Change this line to use a different installed version
+
+#set SSL Engine
+LW2::init_ssl_engine($CONFIGFILE{'LW_SSL_ENGINE'});
 
 my ($a, $b) = split(/\./, $LW2::VERSION);
 die("- You must use LW2 2.4 or later\n") if ($a != 2 || $b < 4);
@@ -94,19 +72,6 @@ general_config();
 load_databases();
 load_databases('u');
 nprint("- $VARIABLES{'name'} v$VARIABLES{'version'}");
-
-LW2::http_init_request(\%request);
-$request{'whisker'}->{'ssl_save_info'}              = 1;
-$request{'whisker'}->{'lowercase_incoming_headers'} = 1;
-$request{'whisker'}->{'timeout'}                    = $CLI{'timeout'} || 10;
-if (defined $CLI{'evasion'}) { $request{'whisker'}->{'encode_anti_ids'} = $CLI{'evasion'}; }
-$request{'User-Agent'} = $VARIABLES{'useragent'};
-$request{'whisker'}->{'retry'} = 0;
-if ($CLI{'useproxy'} && ($CONFIGFILE{PROXYPORT} ne '') && ($CONFIGFILE{PROXYHOST} ne '')) {
-    $request{'whisker'}->{'proxy_host'} = $CONFIGFILE{PROXYHOST};
-    $request{'whisker'}->{'proxy_port'} = $CONFIGFILE{PROXYPORT};
-}
-
 nprint($VARIABLES{'DIV'});
 
 # No targets - quit while we're ahead
@@ -121,6 +86,11 @@ load_plugins();
 # Parse the supplied list of targets
 my @MARKS = set_targets($CLI{'host'}, $CLI{'ports'}, $CLI{'ssl'}, $CLI{'root'});
 
+if (defined($CLI{'key'}) || defined($CLI{'cert'})) {
+    $CLI{'key'}  = $CLI{'cert'} unless (defined($CLI{'key'}));
+    $CLI{'cert'} = $CLI{'key'}  unless (defined($CLI{'cert'}));
+}
+
 # Now check each target is real and remove duplicates/fill in extra information
 foreach my $mark (@MARKS) {
     $mark->{'test'} = 1;
@@ -135,7 +105,8 @@ foreach my $mark (@MARKS) {
     }
 
     # Check that the port is open
-    my $open = port_check($mark->{'hostname'}, $mark->{'ip'}, $mark->{'port'});
+    my $open =
+      port_check($mark->{'hostname'}, $mark->{'ip'}, $mark->{'port'}, $CLI{'key'}, $CLI{'cert'});
     if (defined $CLI{'vhost'}) { $mark->{'vhost'} = $CLI{'vhost'} }
     if ($open == 0) {
         $mark->{'test'} = 0;
@@ -146,6 +117,10 @@ foreach my $mark (@MARKS) {
     }
     $mark->{'ssl'} = $open - 1;
 
+    if ($mark->{'ssl'}) {
+        $mark->{'key'}  = $CLI{'key'};
+        $mark->{'cert'} = $CLI{'cert'};
+    }
 }
 
 # Open reporting
@@ -163,15 +138,15 @@ foreach my $mark (@MARKS) {
     $mark->{'start_time'} = time();
     $VARIABLES{'TEMPL_HCTR'}++;
 
-    # These should just be passed in the hash - but it's a lot of work to move to a local $request
-    $request{'whisker'}->{'host'} = $mark->{'hostname'} || $mark->{'ip'};
     if (defined $CLI{'vhost'}) {
-        $request{'Host'} = $CLI{'vhost'};
         $mark->{'vhost'} = $CLI{'vhost'};
     }
-    $request{'whisker'}->{'port'}    = $mark->{'port'};
-    $request{'whisker'}->{'ssl'}     = $mark->{'ssl'};
-    $request{'whisker'}->{'version'} = $CONFIGFILE{'DEFAULTHTTPVER'};
+
+    # Saving responses
+    if ($CLI{'saveresults'} ne '') {
+        $mark->{'save_dir'} = save_createdir($CLI{'saveresults'}, $mark);
+        $mark->{'save_prefix'} = save_getprefix($mark);
+    }
 
     # Cookies
     if (defined $CONFIGFILE{'STATIC-COOKIE'}) {
@@ -193,11 +168,10 @@ foreach my $mark (@MARKS) {
 
     my %FoF = ();
 
-    nfetch($mark, "/", "GET", "", "", { nocache => 1, noprefetch => 1, nopostfetch => 1 },
-           "getinfo");
+    nfetch($mark, "/", "GET", "", "", { noprefetch => 1, nopostfetch => 1 }, "getinfo");
 
     report_host_start($mark);
-    if ($CLI{'plugins'} eq '@@NONE') {
+    if ($CLI{'findonly'}) {
         my $protocol = "http";
         if ($mark->{'ssl'}) { $protocol .= "s"; }
         if ($mark->{'banner'} eq "") {
@@ -215,9 +189,8 @@ foreach my $mark (@MARKS) {
         run_hooks($mark, "scan");
     }
     $mark->{'end_time'} = time();
-    my $time    = date_disp($mark->{'end_time'});
-    my $elapsed = $mark->{'end_time'} - $mark->{'start_time'};
-    if ($CLI{'plugins'} ne '@@NONE') {
+    $mark->{'elapsed'}  = $mark->{'end_time'} - $mark->{'start_time'};
+    if (!$CLI{'findonly'}) {
         if (!$mark->{'terminate'}) {
             nprint(
                 "+ $COUNTERS{'total_checks'} items checked: $mark->{'total_errors'} error(s) and $mark->{'total_vulns'} item(s) reported on remote host"
@@ -228,7 +201,9 @@ foreach my $mark (@MARKS) {
                 "+ Scan terminated:  $mark->{'total_errors'} error(s) and $mark->{'total_vulns'} item(s) reported on remote host"
                 );
         }
-        nprint("+ End Time:           $time ($elapsed seconds)");
+        nprint(  "+ End Time:           "
+               . date_disp($mark->{'end_time'})
+               . " (GMT$VARIABLES{'GMTOFFSET'}) ($mark->{'elapsed'} seconds)");
     }
     nprint($VARIABLES{'DIV'});
 
@@ -240,16 +215,60 @@ $COUNTERS{'scan_elapsed'} = ($COUNTERS{'scan_end'} - $COUNTERS{'scan_start'});
 report_summary();
 report_close();
 
-if ($CLI{'plugins'} ne '@@NONE') {
+if (!$CLI{'findonly'}) {
     nprint("+ $COUNTERS{'hosts_completed'} host(s) tested");
     nprint("+ $COUNTERS{'totalrequests'} requests made in $COUNTERS{'scan_elapsed'} seconds", "v");
 
     send_updates(@MARKS);
 }
+
 nprint("T:" . localtime() . ": Ending", "d");
 
 exit;
 
+#################################################################################
+# Load config files in order
+sub config_init {
+
+    # read just the --config option
+    {
+        my %optcfg;
+        Getopt::Long::Configure('pass_through', 'noauto_abbrev');
+        GetOptions(\%optcfg, "config=s");
+        Getopt::Long::Configure('nopass_through', 'auto_abbrev');
+        if (defined $optcfg{'config'}) { $VARIABLES{'configfile'} = $optcfg{'config'}; }
+    }
+
+    # Read the config files in order
+    my ($error, $home);
+    my $config_exists = 0;
+    $error = load_config("$VARIABLES{'configfile'}");
+    $config_exists = 1 if ($error eq "");
+
+    # Guess home directory -- to support Windows
+    foreach my $var (split(/ /, "HOME USERPROFILE")) {
+        $home = $ENV{$var} if ($ENV{$var});
+    }
+    $error = load_config("$home/nikto.conf");
+    $config_exists = 1 if ($error eq "");
+
+    # Guess Nikto current directory
+    my $NIKTODIR = $0;
+    chomp($NIKTODIR);
+    $NIKTODIR =~ s#[\\/]nikto.pl$##;
+    $error = load_config("$NIKTODIR/nikto.conf");
+    $config_exists = 1 if ($error eq "");
+
+    $error = load_config("nikto.conf");
+    $config_exists = 1 if ($error eq "");
+
+    if ($config_exists == 0) {
+        die "- Could not find a valid nikto config file\n";
+    }
+
+    return;
+}
+
 #################################################################################
 # load config file
 # error=load_config(FILENAME)
@@ -289,6 +308,7 @@ sub setup_dirs {
     my $CURRENTDIR = $0;
     chomp($CURRENTDIR);
     $CURRENTDIR =~ s#[\\/]nikto.pl$##;
+    $CURRENTDIR = "." if $CURRENTDIR =~ /^nikto.pl$/;
 
     # First assume we get it from CONFIGFILE
     unless (defined $CONFIGFILE{'EXECDIR'}) {
@@ -315,6 +335,9 @@ sub setup_dirs {
     unless (defined $CONFIGFILE{'DOCUMENTDIR'}) {
         $CONFIGFILE{'DOCUMENTDIR'} = "$CONFIGFILE{'EXECDIR'}/docs";
     }
+    unless (defined $CONFIGFILE{'DBDIR'}) {
+        $CONFIGFILE{'DBDIR'} = "$CONFIGFILE{'EXECDIR'}/databases";
+    }
     return;
 }
 
@@ -330,4 +353,6 @@ sub check_config_defined {
           "- Warning: $item is not defined in Nikto configuration, setting to \"$default\"\n";
         $CONFIGFILE{$item} = $default;
     }
+
+    return;
 }
